
#include <stdio.h>
#include <stdlib.h>
#include "error.h"


#include "mfhdf.h"




int report_error(int code, int line, char *file)
{
	char msg[256];

	/* skip file/location information if zero line number or null string */
	if (file && (line > 0)) {
		(void) sprintf(msg, "*** FATAL ERROR (file \"%s\", line %d):",
			file, line);
		errmsg(msg);
		}


	switch (code) {
		case ERR_HDF:
			errmsg("HDF library error.");
			HEprint(stderr, 0);
			break;

		case ERR_HDF_QUIET:
			/* like ERR_HDF except don't print HDF error message */
			break;

		case ERR_F_MEM:
			errmsg("Unable to allocate memory.");
			break;

		case ERR_DATA_TYPE:
			errmsg("Unexpected data type within HDF file.");
			break;

		case ERR_DATA_RANK:
			errmsg("Unexpected data rank within HDF file.");
			break;

		case ERR_DATA_SIZE:
			errmsg("Unexpected data size within HDF file.");
			break;

		case ERR_MISSING_ATTR:
			errmsg("Missing SDS attribute in HDF file.");
			break;

		case ERR_IMAGE_LINES:
			errmsg("Number of lines in granule not a multiple of 10.");
			break;

		case ERR_NLINES_MISMATCH:
			errmsg("Number of scans in VMAE and QSLWMIP files do not match.");
			break;
		}

	return code;
}



void report_warn(char *msg, int line, char *file)
{
	char s[256];

	/* skip file/location information if zero line number or null string */
	if (file && (line > 0)) {
		(void) sprintf(s, "*** WARNING (file \"%s\", line %d):",
			file, line);
		errmsg(s);
		}

	errmsg(msg);
}



/* adds newline to error message so caller doesn't need to provide it */
void errmsg(char *msg)
{
	(void) fputs(msg, stderr);
	(void) fputc('\n', stderr);
}


