#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>

#include "misc.h"


/*
This is a bit of a kludge since there seems to be no POSIX
standard for the maximum size of the strings in the utsname
structure.  On some systems the length of each field is SYS_NMLN,
but under Linux and other operating systems it's called
something else.  We'll be safe using the total size of the
structure, however, at the expense of a miniscule amount of wasted
memory.
*/
#define MAX_UNAME_LEN sizeof(struct utsname)


char *make_uname(void)
{
	struct utsname name;
	static char unamestr[MAX_UNAME_LEN];

	(void) uname(&name);

	/* Note: space for the whitespace and terminating null is
	guaranteed because each of the five string fields we're
	extracting has a terminating null.  This guarantees at least
	five unused characters in unamestr[].  If additional printing
	characters are added to the format, however, add the same
	number of characters in the definition of MAX_UNAME_LEN above. */
	sprintf(unamestr, "%s %s %s %s %s", name.sysname, name.nodename,
		name.release, name.version, name.machine);

	return unamestr;
}



