/*
!C******************************************************************
!Description:

	Functions related to monitoring QF and BT.

!Revision History:
!Team-unique Header:
!References and Credits:

	W. Walsh, May 2014

!END*************************************************************************/

#include "mfhdf.h"
#include "const.h"
#include "types.h"
#include <stdio.h>

#define ROWS_QF 6
#define COLS_QF 256

#define ROWS_BT 3
#define COLS_BT 699

static int init = 0;
static int32 qf_buffer[ROWS_QF][COLS_QF];
static int32 bt_buffer[ROWS_BT][COLS_BT][2];

void update_monitor(Scan *Ptr)
{
	int row, col;
	int value;
	
	/* Initialize the buffers to zero */
	if (init == 0)
	{
		init = 1;

		for (row=0;row<ROWS_QF;row++)
		{
			for (col=0;col<COLS_QF;col++)
			{
				qf_buffer[row][col] = 0;
			}
		}

		for (row=0;row<ROWS_BT;row++)
		{
			for (col=0;col<COLS_BT;col++)
			{
				bt_buffer[row][col][0] = 0;
				bt_buffer[row][col][1] = 0;
			}
		}

	}

	for (row=0;row<LINES_PER_SCAN_750M;row++)
	{
		for (col=0;col<SAMPLES_750M;col++)
		{
			/* QF5 */
			value = Ptr->QF5[row][col];
			qf_buffer[0][value]++;
			
			/* QF7 */
			value = Ptr->QF7[row][col];
			qf_buffer[1][value]++;

			/* QF11 */
			value = Ptr->QF11[row][col];
			qf_buffer[2][value]++;

			/* QF13 */
			value = Ptr->QF13[row][col];
			qf_buffer[3][value]++;

			/* T13 */
			if (value == 0){
				value = Ptr->T13[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[0][value][0]++;
			}else{
				value = Ptr->T13[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[0][value][1]++;
			}

			/* QF15 */
			value = Ptr->QF15[row][col];
			qf_buffer[4][value]++;

			/* T15 */
			if (value == 0){
				value = Ptr->T15[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[1][value][0]++;
			}else{
				value = Ptr->T15[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[1][value][1]++;
			}
	
			/* QF16 */
			value = Ptr->QF15[row][col];
			qf_buffer[5][value]++;

			/* T16 */
			if (value == 0){
				value = Ptr->T16[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[2][value][0]++;
			}else{
				value = Ptr->T16[row][col];
				if (value < 0) value = 0;
				if (value >= COLS_BT) value = COLS_BT;
				bt_buffer[2][value][1]++;
			}


		}
	}
}


int write_monitor(int file_id)
{
	int sds_id, dim_id, rt;
	int qf_dims[2], qf_start[2], qf_edges[2];
	int bt_dims[3], bt_start[3], bt_edges[3];
	int rank = 2;
comp_info c_info;


	/*** qf counter ***/
	qf_dims[0]=ROWS_QF;
	qf_dims[1]=COLS_QF;

	/* Create the array data set. */
	sds_id = SDcreate(file_id, "qf counter", DFNT_INT32, rank, qf_dims);
	if (sds_id == FAIL) return 1;

	c_info.deflate.level = 4;
	if (SDsetcompress(sds_id, COMP_CODE_DEFLATE, &c_info) == FAIL) return 2;


	/* Name dimensions */
	dim_id = SDgetdimid(sds_id, 0);
	SDsetdimname(dim_id, "YDim:Band");

	dim_id = SDgetdimid(sds_id, 1);
	SDsetdimname(dim_id, "XDim:QF Value");


	qf_start[0] = 0;
	qf_start[1] = 0;
	qf_edges[0] = ROWS_QF;
	qf_edges[1] = COLS_QF;

	

	rt = SDwritedata(sds_id, qf_start, NULL, qf_edges, (VOIDP)qf_buffer);
	if (rt != 0) return rt;

	/* Terminate access to the array. */
	rt = SDendaccess(sds_id);
	if (rt != 0) return rt;

	/***  ***/


	/*** bt counter ***/
	rank = 3;
	bt_dims[0]=ROWS_BT;
	bt_dims[1]=COLS_BT;
	bt_dims[2]=2;

	/* Create the array data set. */
	sds_id = SDcreate(file_id, "bt counter", DFNT_INT32, rank, bt_dims);
	if (sds_id == FAIL) return 1;

	/* Name dimensions */
	dim_id = SDgetdimid(sds_id, 0);
	SDsetdimname(dim_id, "YDim: Bands");

	dim_id = SDgetdimid(sds_id, 1);
	SDsetdimname(dim_id, "XDim: Brightness Temp");

	dim_id = SDgetdimid(sds_id, 2);
	SDsetdimname(dim_id, "ZDim: QF (zero/non-zero)");


	bt_start[0] = 0;
	bt_start[1] = 0;
	bt_start[2] = 0;
	bt_edges[0] = ROWS_BT;
	bt_edges[1] = COLS_BT;
	bt_edges[2] = 2;


	rt = SDwritedata(sds_id, bt_start, NULL, bt_edges, (VOIDP)bt_buffer);
	if (rt != 0) return rt;

	/* Terminate access to the array. */
	rt = SDendaccess(sds_id);
	if (rt != 0) return rt;

	/***  ***/

	return 0;
}
