#ifndef TYPES_H
#define TYPES_H

typedef unsigned char UBYTE;
typedef unsigned short USHORT;

enum Satellite {SAT_NPP, SAT_UNKNOWN};
enum {WATER, COAST, LAND};


/* pixel-level flags and values used internally by algorithm */
typedef struct {
	unsigned trim		: 1;	/* flag: trimmed EDR pixel */
	unsigned day		: 1;	/* daytime = 1, nighttime = 0 */
	unsigned potential_fire	: 1;	/* flag: potential fire pixel */
	unsigned missing_rad	: 1;	/* flag: missing radiance data */
	unsigned missing_geo	: 1;	/* flag: bad geolocation */
	unsigned missing_lc	: 1;	/* flag: missing land cover */
	unsigned surf		: 2;	/* surface type */
	unsigned cloud		: 1;	/* flag: cloudy pixel */
	unsigned process	: 1;	/* flag: apply detection algorithm
						to pixel */
	unsigned zsafe		: 1;	/* flag: include pixel when computing
						zone statistics */

	unsigned valid_bkg_pix	: 1;	/* flag: valid background pixel */
	unsigned bkg_fire	: 1;	/* flag: probable background-fire
						pixel */
	unsigned glint_level	: 2;	/* sun glint level (0-3) */
	} PixInfo;


typedef struct {
	float32	T13[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	T15[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	T16[LINES_PER_SCAN_750M][SAMPLES_750M];

	float32	R5[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	R7[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	R11[LINES_PER_SCAN_750M][SAMPLES_750M];

	uint8	QF5[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8	QF7[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8	QF11[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8	QF13[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8	QF15[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8	QF16[LINES_PER_SCAN_750M][SAMPLES_750M];

	float32	lat[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	lon[LINES_PER_SCAN_750M][SAMPLES_750M];

	float32	SolZenAng[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	SolAzAng[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	ViewZenAng[LINES_PER_SCAN_750M][SAMPLES_750M];
	float32	ViewAzAng[LINES_PER_SCAN_750M][SAMPLES_750M];

	uint8	landsea[LINES_PER_SCAN_750M][SAMPLES_750M];

	/* used internally by algorithm */
	PixInfo	pixflags[LINES_PER_SCAN_750M][SAMPLES_750M];

	float32	LandMeanT13[SAMPLES_750M], LandMeanDT[SAMPLES_750M];
	uint16	LandZonePix[SAMPLES_750M];
	float32	WaterMeanT13[SAMPLES_750M], WaterMeanDT[SAMPLES_750M];
	uint16	WaterZonePix[SAMPLES_750M];

	float32	L13[LINES_PER_SCAN_750M][SAMPLES_750M];
	} Scan;


typedef struct {
	float thresh_PF_T13[LINES_PER_SCAN_750M][SAMPLES_750M];
	float thresh_PF_DT[LINES_PER_SCAN_750M][SAMPLES_750M];
	} ScanThreshPF;



typedef struct {
	uint8   FireMask[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint8   ExtFireMask[LINES_PER_SCAN_750M][SAMPLES_750M];
	uint32  FireQA[LINES_PER_SCAN_750M][SAMPLES_750M];
	} OutScan;



typedef struct{
	int32 NumPix;
  
	int32 NumMissingPix;
	int32 NumMissingRad;
	int32 NumMissingGeo;
	int32 NumTrimPix;
	int32 NumLandPix;
	int32 NumFirePix;
	int32 NumLandFirePix;
	int32 NumWaterFirePix;

	int32 NumWaterPix;
	int32 NumUnknownLandPix;
	int32 NumUnknownWaterPix;
	int32 NumNoFirePix;
	int32 NumGlintPix;
	int32 NumGlintRejectPix;
	int32 NumCoastRejectLandPix;
	int32 NumHotRejectPix;
	int32 NumClearingRejectPix;

	int32 NumCoastRejectWaterPix;

	int32 NumWaterAdjFirePix;
	int32 NumCloudAdjFirePix;

	int32 NumWaterCloudPix;
	int32 NumLandCloudPix;

	int32 NumDayPix;
	int32 NumNightPix;

	int32 NumGoodQuality;
	int32 NumOtherQuality;
	int32 NumNotProducedCloud;
	int32 NumNotProducedOther;
	} FireCounts;



#endif
